@@ -1,4 +1,4 @@
-CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, standardize, twostep, ...)
+CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, standardize, twostep, sample.weights, ...)
 {
   probs.min<-1e-6
   
@@ -8,6 +8,9 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   T2<-as.numeric(treat==treat.names[2])
   T3<-as.numeric(treat==treat.names[3])
   
+  sample.weights<-sample.weights/mean(sample.weights)
+  XprimeX.inv<-ginv(t(sample.weights^.5*X)%*%(sample.weights^.5*X))
+  
   ##The gmm objective function--given a guess of beta, constructs the GMM J statistic.
   gmm.func<-function(beta.curr,X.gmm=X,invV=NULL){
     ##Designate a few objects in the function.
@@ -31,13 +34,13 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
     
     
     ##Generate the vector of mean imbalance by weights.
-    w.curr.del<-1/n*t(X)%*%w.curr
+    w.curr.del<-1/n*t(sample.weights*X)%*%w.curr
     w.curr.del<-as.matrix(w.curr.del)
     w.curr<-as.matrix(w.curr)
     
     ##Generate g-bar, as in the paper.
-    gbar<-c(1/n*t(X)%*%(T2-probs.curr[,2]), 
-            1/n*t(X)%*%(T3-probs.curr[,3]),
+    gbar<-c(1/n*t(sample.weights*X)%*%(T2-probs.curr[,2]), 
+            1/n*t(sample.weights*X)%*%(T3-probs.curr[,3]),
             w.curr.del)
     
     
@@ -45,16 +48,16 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
     {
       ##Generate the covariance matrix used in the GMM estimate.
       ##Was for the initial version that calculates the analytic variances.    
-      X.1.1<-X*(probs.curr[,2]*(1-probs.curr[,2]))
-      X.1.2<-X*(-probs.curr[,2]*probs.curr[,3])
-      X.1.3<-X*-1
-      X.1.4<-X*1
-      X.2.2<-X*(probs.curr[,3]*(1-probs.curr[,3]))
-      X.2.3<-X*-1
-      X.2.4<-X*-1
-      X.3.3<-X*(4*probs.curr[,1]^-1+probs.curr[,2]^-1+probs.curr[,3]^-1)
-      X.3.4<-X*(-probs.curr[,2]^-1+probs.curr[,3]^-1)
-      X.4.4<-X*(probs.curr[,2]^-1+probs.curr[,3]^-1)
+      X.1.1<-sample.weights*X*(probs.curr[,2]*(1-probs.curr[,2]))
+      X.1.2<-sample.weights*X*(-probs.curr[,2]*probs.curr[,3])
+      X.1.3<-sample.weights*X*-1
+      X.1.4<-sample.weights*X*1
+      X.2.2<-sample.weights*X*(probs.curr[,3]*(1-probs.curr[,3]))
+      X.2.3<-sample.weights*X*-1
+      X.2.4<-sample.weights*X*-1
+      X.3.3<-sample.weights*X*(4*probs.curr[,1]^-1+probs.curr[,2]^-1+probs.curr[,3]^-1)
+      X.3.4<-sample.weights*X*(-probs.curr[,2]^-1+probs.curr[,3]^-1)
+      X.4.4<-sample.weights*X*(probs.curr[,2]^-1+probs.curr[,3]^-1)
       
       V<-1/n*rbind(cbind(t(X.1.1)%*%X,t(X.1.2)%*%X,t(X.1.3)%*%X,t(X.1.4)%*%X),
                    cbind(t(X.1.2)%*%X,t(X.2.2)%*%X,t(X.2.3)%*%X,t(X.2.4)%*%X),
@@ -87,7 +90,7 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
                   T2/probs.curr[,2] - T3/probs.curr[,3])/n
     
     ##Generate mean imbalance.
-    loss1<-sum(diag(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr)))
+    loss1<-sum(diag(t(w.curr)%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr)))
     loss1
   }
   
@@ -114,23 +117,23 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
     
     
     ##Generate the vector of mean imbalance by weights.
-    w.curr.del<-1/n*t(X)%*%w.curr
+    w.curr.del<-1/n*t(sample.weights*X)%*%w.curr
     w.curr.del<-as.matrix(w.curr.del)
     w.curr<-as.matrix(w.curr)
     
     ##Generate g-bar, as in the paper.
-    gbar<-c(1/n*t(X)%*%(T2-probs.curr[,2]), 
-            1/n*t(X)%*%(T3-probs.curr[,3]),
+    gbar<-c(1/n*t(sample.weights*X)%*%(T2-probs.curr[,2]), 
+            1/n*t(sample.weights*X)%*%(T3-probs.curr[,3]),
             w.curr.del)
     
-    dgbar<-rbind(cbind(1/n*t(-X*probs.curr[,2]*(1-probs.curr[,2]))%*%X,
-                       1/n*t(X*probs.curr[,2]*probs.curr[,3])%*%X,
-                       1/n*t(X*(2*T1*probs.curr[,2]/probs.curr[,1] + T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3]))%*%X,
-                       1/n*t(X*(-T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3]))%*%X),
-                 cbind(1/n*t(X*probs.curr[,2]*probs.curr[,3])%*%X,
-                       1/n*t(-X*probs.curr[,3]*(1-probs.curr[,3]))%*%X,
-                       1/n*t(X*(2*T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3]))%*%X,
-                       1/n*t(X*(T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3]))%*%X))
+    dgbar<-rbind(cbind(1/n*t(-sample.weights*X*probs.curr[,2]*(1-probs.curr[,2]))%*%X,
+                       1/n*t(sample.weights*X*probs.curr[,2]*probs.curr[,3])%*%X,
+                       1/n*t(sample.weights*X*(2*T1*probs.curr[,2]/probs.curr[,1] + T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3]))%*%X,
+                       1/n*t(sample.weights*X*(-T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3]))%*%X),
+                 cbind(1/n*t(sample.weights*X*probs.curr[,2]*probs.curr[,3])%*%X,
+                       1/n*t(-sample.weights*X*probs.curr[,3]*(1-probs.curr[,3]))%*%X,
+                       1/n*t(sample.weights*X*(2*T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3]))%*%X,
+                       1/n*t(sample.weights*X*(T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3]))%*%X))
                  
     out<-2*dgbar%*%invV%*%gbar
     out
@@ -151,16 +154,16 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
     w.curr<-cbind(2*T1/probs.curr[,1] - T2/probs.curr[,2] - T3/probs.curr[,3],
                   T2/probs.curr[,2] - T3/probs.curr[,3])/n
     
-    dw.beta1<-cbind(t(X*(2*T1*probs.curr[,2]/probs.curr[,1] + T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3])),
-                    t(X*(-T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3])))/n
-    dw.beta2<-cbind(t(X*(2*T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3])),
-                    t(X*(T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3])))/n
+    dw.beta1<-cbind(t(sample.weights*X*(2*T1*probs.curr[,2]/probs.curr[,1] + T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3])),
+                    t(sample.weights*X*(-T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3])))/n
+    dw.beta2<-cbind(t(sample.weights*X*(2*T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3])),
+                    t(sample.weights*X*(T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3])))/n
     ##Generate mean imbalance.
-    loss1<-diag(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr))
-    out.1<-2*dw.beta1[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) + 
-      2*dw.beta1[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2])
-    out.2<-2*dw.beta2[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) +
-      2*dw.beta2[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2])
+    loss1<-diag(t(w.curr)%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr))
+    out.1<-2*dw.beta1[,1:n]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,1]) + 
+      2*dw.beta1[,(n+1):(2*n)]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,2])
+    out.2<-2*dw.beta2[,1:n]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,1]) +
+      2*dw.beta2[,(n+1):(2*n)]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,2])
     out<-c(out.1, out.2)
     out
   }
@@ -172,7 +175,7 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   #Need to generalize for different dimensioned X's
   xnam<- colnames(dat.dummy[,-1])
   fmla <- as.formula(paste("as.factor(treat) ~ -1 + ", paste(xnam, collapse= "+")))
-  mnl1<-multinom(fmla, data=dat.dummy, trace=FALSE)
+  mnl1<-multinom(fmla, data=dat.dummy, weights = sample.weights, trace=FALSE)
   mcoef<-t(coef(mnl1))
   mcoef[is.na(mcoef[,1]),1]<-0
   mcoef[is.na(mcoef[,2]),2]<-0
@@ -272,28 +275,28 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   norm1<-norm2<-norm3<-1
   if (standardize)
   {
-    norm1<-sum(T1/probs.opt[,1])
-    norm2<-sum(T2/probs.opt[,2])
-    norm3<-sum(T3/probs.opt[,3])
+    norm1<-sum(T1*sample.weights/probs.opt[,1])
+    norm2<-sum(T2*sample.weights/probs.opt[,2])
+    norm3<-sum(T3*sample.weights/probs.opt[,3])
   }
   w.opt<-T1/probs.opt[,1]/norm1 + T2/probs.opt[,2]/norm2 + T3/probs.opt[,3]/norm3
   
   W<-gmm.func(beta.opt)$invV
 
-  XG.1.1<-t(-X*probs.opt[,2]*(1-probs.opt[,2]))%*%X
-  XG.1.2<-t(X*probs.opt[,2]*probs.opt[,3])%*%X
-  XG.1.3<-t(X*(2*T1*probs.opt[,2]/probs.opt[,1] + T2*(1-probs.opt[,2])/probs.opt[,2] - T3*probs.opt[,2]/probs.opt[,3]))%*%X
-  XG.1.4<-t(X*(-T2*(1-probs.opt[,2])/probs.opt[,2] - T3*probs.opt[,2]/probs.opt[,3]))%*%X
-  XG.2.1<-t(X*probs.opt[,2]*probs.opt[,3])%*%X
-  XG.2.2<-t(-X*probs.opt[,3]*(1-probs.opt[,3]))%*%X
-  XG.2.3<-t(X*(2*T1*probs.opt[,3]/probs.opt[,1] - T2*probs.opt[,3]/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]))%*%X
-  XG.2.4<-t(X*(T2*probs.opt[,3]/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]))%*%X
+  XG.1.1<-t(-X*sample.weights*probs.opt[,2]*(1-probs.opt[,2]))%*%X
+  XG.1.2<-t(X*sample.weights*probs.opt[,2]*probs.opt[,3])%*%X
+  XG.1.3<-t(X*sample.weights*(2*T1*probs.opt[,2]/probs.opt[,1] + T2*(1-probs.opt[,2])/probs.opt[,2] - T3*probs.opt[,2]/probs.opt[,3]))%*%X
+  XG.1.4<-t(X*sample.weights*(-T2*(1-probs.opt[,2])/probs.opt[,2] - T3*probs.opt[,2]/probs.opt[,3]))%*%X
+  XG.2.1<-t(X*sample.weights*probs.opt[,2]*probs.opt[,3])%*%X
+  XG.2.2<-t(-X*sample.weights*probs.opt[,3]*(1-probs.opt[,3]))%*%X
+  XG.2.3<-t(X*sample.weights*(2*T1*probs.opt[,3]/probs.opt[,1] - T2*probs.opt[,3]/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]))%*%X
+  XG.2.4<-t(X*sample.weights*(T2*probs.opt[,3]/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]))%*%X
   G<-1/n*rbind(cbind(XG.1.1,XG.1.2,XG.1.3,XG.1.4),cbind(XG.2.1,XG.2.2,XG.2.3,XG.2.4))
   
-  XW.1<-X*(T2-probs.opt[,2])
-  XW.2<-X*(T3-probs.opt[,3])
-  XW.3<-X*(2*T1/probs.opt[,1] - T2/probs.opt[,2] - T3/probs.opt[,3])
-  XW.4<-X*(T2/probs.opt[,2] - T3/probs.opt[,3])
+  XW.1<-X*(T2-probs.opt[,2])*sample.weights^.5
+  XW.2<-X*(T3-probs.opt[,3])*sample.weights^.5
+  XW.3<-X*(2*T1/probs.opt[,1] - T2/probs.opt[,2] - T3/probs.opt[,3])*sample.weights^.5
+  XW.4<-X*(T2/probs.opt[,2] - T3/probs.opt[,3])*sample.weights^.5
   W1<-rbind(t(XW.1),t(XW.2),t(XW.3),t(XW.4))
   Omega<-1/n*(W1%*%t(W1))
   vcov<-ginv(G%*%W%*%t(G))%*%G%*%W%*%Omega%*%W%*%t(G)%*%ginv(G%*%W%*%t(G))
@@ -302,7 +305,7 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   
   class(beta.opt) <- "coef"
   
-  output<-list("coefficients"=beta.opt,"fitted.values"=probs.opt,"deviance"=deviance,"weights"=w.opt, 
+  output<-list("coefficients"=beta.opt,"fitted.values"=probs.opt,"deviance"=deviance,"weights"=w.opt*sample.weights, 
                "y"=treat,"x"=X,"converged"=opt1$conv,"J"=J.opt,"var"=vcov, 
                "mle.J"=ifelse(twostep, gmm.func(mcoef, invV = this.invV)$loss, gmm.loss(mcoef)))
   
@@ -311,7 +314,7 @@ CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   output
 }
 
-CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, standardize, twostep, ...)
+CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, standardize, twostep, sample.weights, ...)
 {
   probs.min<-1e-6
   
@@ -322,6 +325,9 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   T3<-as.numeric(treat==treat.names[3])
   T4<-as.numeric(treat==treat.names[4])
   
+  sample.weights<-sample.weights/mean(sample.weights)
+  XprimeX.inv<-ginv(t(sample.weights^.5*X)%*%(sample.weights^.5*X))
+  
   ##The gmm objective function--given a guess of beta, constructs the GMM J statistic.
   gmm.func<-function(beta.curr,X.gmm=X,invV=NULL){
     ##Designate a few objects in the function.
@@ -346,14 +352,14 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
                   -T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4])    
     
     ##Generate the vector of mean imbalance by weights.
-    w.curr.del<-1/n*t(X)%*%w.curr
+    w.curr.del<-1/n*t(sample.weights*X)%*%w.curr
     w.curr.del<-as.matrix(w.curr.del)
     w.curr<-as.matrix(w.curr)
     
     ##Generate g-bar, as in the paper.
-    gbar<-c(1/n*t(X)%*%(T2-probs.curr[,2]), 
-            1/n*t(X)%*%(T3-probs.curr[,3]),
-            1/n*t(X)%*%(T4-probs.curr[,4]),
+    gbar<-c(1/n*t(sample.weights*X)%*%(T2-probs.curr[,2]), 
+            1/n*t(sample.weights*X)%*%(T3-probs.curr[,3]),
+            1/n*t(sample.weights*X)%*%(T4-probs.curr[,4]),
             w.curr.del)
     
     
@@ -361,24 +367,24 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
     {
       ##Generate the covariance matrix used in the GMM estimate.
       ##Was for the initial version that calculates the analytic variances.		
-      X.1.1<-X*(probs.curr[,2]*(1-probs.curr[,2]))
-      X.1.2<-X*(-probs.curr[,2]*probs.curr[,3])
-      X.1.3<-X*(-probs.curr[,2]*probs.curr[,4])
-      X.1.4<-X
-      X.1.5<-X*(-1)
-      X.1.6<-X
-      X.2.2<-X*(probs.curr[,3]*(1-probs.curr[,3]))
-      X.2.3<-X*(-probs.curr[,3]*probs.curr[,4])
-      X.2.4<-X*(-1)
-      X.2.5<-X*(-1)
-      X.2.6<-X*(-1)
-      X.3.3<-X*(probs.curr[,4]*(1-probs.curr[,4]))
-      X.3.4<-X*(-1)
-      X.3.5<-X
-      X.3.6<-X
-      X.4.4<-X*(probs.curr[,1]^-1+probs.curr[,2]^-1+probs.curr[,3]^-1+probs.curr[,4]^-1)
-      X.4.5<-X*(probs.curr[,1]^-1-probs.curr[,2]^-1+probs.curr[,3]^-1-probs.curr[,4]^-1)
-      X.4.6<-X*(-probs.curr[,1]^-1+probs.curr[,2]^-1+probs.curr[,3]^-1-probs.curr[,4]^-1)
+      X.1.1<-sample.weights*X*(probs.curr[,2]*(1-probs.curr[,2]))
+      X.1.2<-sample.weights*X*(-probs.curr[,2]*probs.curr[,3])
+      X.1.3<-sample.weights*X*(-probs.curr[,2]*probs.curr[,4])
+      X.1.4<-sample.weights*X
+      X.1.5<-sample.weights*X*(-1)
+      X.1.6<-sample.weights*X
+      X.2.2<-sample.weights*X*(probs.curr[,3]*(1-probs.curr[,3]))
+      X.2.3<-sample.weights*X*(-probs.curr[,3]*probs.curr[,4])
+      X.2.4<-sample.weights*X*(-1)
+      X.2.5<-sample.weights*X*(-1)
+      X.2.6<-sample.weights*X*(-1)
+      X.3.3<-sample.weights*X*(probs.curr[,4]*(1-probs.curr[,4]))
+      X.3.4<-sample.weights*X*(-1)
+      X.3.5<-sample.weights*X
+      X.3.6<-sample.weights*X
+      X.4.4<-sample.weights*X*(probs.curr[,1]^-1+probs.curr[,2]^-1+probs.curr[,3]^-1+probs.curr[,4]^-1)
+      X.4.5<-sample.weights*X*(probs.curr[,1]^-1-probs.curr[,2]^-1+probs.curr[,3]^-1-probs.curr[,4]^-1)
+      X.4.6<-sample.weights*X*(-probs.curr[,1]^-1+probs.curr[,2]^-1+probs.curr[,3]^-1-probs.curr[,4]^-1)
       X.5.5<-X.4.4
       X.5.6<-X*(-probs.curr[,1]^-1-probs.curr[,2]^-1+probs.curr[,3]^-1+probs.curr[,4]^-1)
       X.6.6<-X.4.4
@@ -416,7 +422,7 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
                   -T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4])/n
     
     ##Generate mean imbalance.
-    loss1<-sum(diag(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr)))
+    loss1<-sum(diag(t(w.curr)%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr)))
     loss1
   }
   
@@ -445,36 +451,36 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
     
     
     ##Generate the vector of mean imbalance by weights.
-    w.curr.del<-1/n*t(X)%*%w.curr
+    w.curr.del<-1/n*t(sample.weights*X)%*%w.curr
     w.curr.del<-as.matrix(w.curr.del)
     w.curr<-as.matrix(w.curr)
     
     ##Generate g-bar, as in the paper.
-    gbar<-c(1/n*t(X)%*%(T2-probs.curr[,2]), 
-            1/n*t(X)%*%(T3-probs.curr[,3]),
-            1/n*t(X)%*%(T4-probs.curr[,4]),
+    gbar<-c(1/n*t(sample.weights*X)%*%(T2-probs.curr[,2]), 
+            1/n*t(sample.weights*X)%*%(T3-probs.curr[,3]),
+            1/n*t(sample.weights*X)%*%(T4-probs.curr[,4]),
             w.curr.del)
     
-    dgbar<-rbind(cbind(1/n*t(-X*probs.curr[,2]*(1-probs.curr[,2]))%*%X,
-                       1/n*t(X*probs.curr[,2]*probs.curr[,3])%*%X,
-                       1/n*t(X*probs.curr[,2]*probs.curr[,4])%*%X,
-                       1/n*t(X*(T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] - T4*probs.curr[,2]/probs.curr[,4]))%*%X,
-                       1/n*t(X*(T1*probs.curr[,2]/probs.curr[,1] + T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))%*%X,
-                       1/n*t(X*(-T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))%*%X
+    dgbar<-rbind(cbind(1/n*t(-sample.weights*X*probs.curr[,2]*(1-probs.curr[,2]))%*%X,
+                       1/n*t(sample.weights*X*probs.curr[,2]*probs.curr[,3])%*%X,
+                       1/n*t(sample.weights*X*probs.curr[,2]*probs.curr[,4])%*%X,
+                       1/n*t(sample.weights*X*(T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] - T4*probs.curr[,2]/probs.curr[,4]))%*%X,
+                       1/n*t(sample.weights*X*(T1*probs.curr[,2]/probs.curr[,1] + T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))%*%X,
+                       1/n*t(sample.weights*X*(-T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))%*%X
     ),
-    cbind(1/n*t(X*probs.curr[,2]*probs.curr[,3])%*%X,
-          1/n*t(-X*probs.curr[,3]*(1-probs.curr[,3]))%*%X,
-          1/n*t(X*probs.curr[,3]*probs.curr[,4])%*%X,
-          1/n*t(X*(T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] - T4*probs.curr[,3]/probs.curr[,4]))%*%X,
-          1/n*t(X*(T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))%*%X,
-          1/n*t(X*(-T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))%*%X
+    cbind(1/n*t(sample.weights*X*probs.curr[,2]*probs.curr[,3])%*%X,
+          1/n*t(-sample.weights*X*probs.curr[,3]*(1-probs.curr[,3]))%*%X,
+          1/n*t(sample.weights*X*probs.curr[,3]*probs.curr[,4])%*%X,
+          1/n*t(sample.weights*X*(T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] - T4*probs.curr[,3]/probs.curr[,4]))%*%X,
+          1/n*t(sample.weights*X*(T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))%*%X,
+          1/n*t(sample.weights*X*(-T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))%*%X
     ),
-    cbind(1/n*t(X*probs.curr[,2]*probs.curr[,4])%*%X,
-          1/n*t(X*probs.curr[,3]*probs.curr[,4])%*%X,
-          1/n*t(-X*probs.curr[,4]*(1-probs.curr[,4]))%*%X,
-          1/n*t(X*(T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] + T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X,
-          1/n*t(X*(T1*probs.curr[,4]/probs.curr[,1] - T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X,
-          1/n*t(X*(-T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X
+    cbind(1/n*t(sample.weights*X*probs.curr[,2]*probs.curr[,4])%*%X,
+          1/n*t(sample.weights*X*probs.curr[,3]*probs.curr[,4])%*%X,
+          1/n*t(-sample.weights*X*probs.curr[,4]*(1-probs.curr[,4]))%*%X,
+          1/n*t(sample.weights*X*(T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] + T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X,
+          1/n*t(sample.weights*X*(T1*probs.curr[,4]/probs.curr[,1] - T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X,
+          1/n*t(sample.weights*X*(-T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X
     ))
     
     out<-2*dgbar%*%invV%*%gbar
@@ -510,17 +516,17 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
                     t(X*(T1*probs.curr[,4]/probs.curr[,1] - T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4])),
                     t(X*(-T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4])))/n
     
-    loss1<-diag(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr))
+    loss1<-diag(t(w.curr)%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr))
     
-    out.1<-2*dw.beta1[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) +
-      2*dw.beta1[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]) +
-      2*dw.beta1[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3])
-    out.2<-2*dw.beta2[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) +
-      2*dw.beta2[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]) +
-      2*dw.beta2[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3])
-    out.3<-2*dw.beta3[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) +
-      2*dw.beta3[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]) +
-      2*dw.beta3[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3])
+    out.1<-2*dw.beta1[,1:n]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,1]) +
+      2*dw.beta1[,(n+1):(2*n)]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,2]) +
+      2*dw.beta1[,(2*n+1):(3*n)]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,3])
+    out.2<-2*dw.beta2[,1:n]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,1]) +
+      2*dw.beta2[,(n+1):(2*n)]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,2]) +
+      2*dw.beta2[,(2*n+1):(3*n)]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,3])
+    out.3<-2*dw.beta3[,1:n]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,1]) +
+      2*dw.beta3[,(n+1):(2*n)]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,2]) +
+      2*dw.beta3[,(2*n+1):(3*n)]%*%(sample.weights*X)%*%XprimeX.inv%*%t(sample.weights*X)%*%(w.curr[,3])
     out<-c(out.1, out.2, out.3)
     out
   }
@@ -532,7 +538,7 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   #Need to generalize for different dimensioned X's
   xnam<- colnames(dat.dummy[,-1])
   fmla <- as.formula(paste("as.factor(treat) ~ -1 + ", paste(xnam, collapse= "+")))
-  mnl1<-multinom(fmla, data=dat.dummy,trace=FALSE)
+  mnl1<-multinom(fmla, data=dat.dummy,trace=FALSE,weights=sample.weights)
   mcoef<-t(coef(mnl1))
   mcoef[is.na(mcoef[,1]),1]<-0
   mcoef[is.na(mcoef[,2]),2]<-0
@@ -653,10 +659,10 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   norm1<-norm2<-norm3<-norm4<-1
   if (standardize)
   {
-    norm1<-sum(1/probs.opt[which(T1==1),1])
-    norm2<-sum(1/probs.opt[which(T2==1),2])
-    norm3<-sum(1/probs.opt[which(T3==1),3])
-    norm4<-sum(1/probs.opt[which(T4==1),4])
+    norm1<-sum((sample.weights/probs.opt)[which(T1==1),1])
+    norm2<-sum((sample.weights/probs.opt)[which(T2==1),2])
+    norm3<-sum((sample.weights/probs.opt)[which(T3==1),3])
+    norm4<-sum((sample.weights/probs.opt)[which(T4==1),4])
   }
   
   w.opt<-(T1 == 1)/probs.opt[,1]/norm1 + (T2 == 1)/probs.opt[,2]/norm2 + (T3 == 1)/probs.opt[,3]/norm3 + (T4 == 1)/probs.opt[,4]/norm4
@@ -664,30 +670,30 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
 
   W<-gmm.func(beta.opt)$invV
   
-  X.G.1.1<-t(-X*probs.opt[,2]*(1-probs.opt[,2]))%*%X
-  X.G.1.2<-t(X*probs.opt[,2]*probs.opt[,3])%*%X
-  X.G.1.3<-t(X*probs.opt[,2]*probs.opt[,4])%*%X
-  X.G.1.4<-t(X*probs.opt[,2]*(T1/probs.opt[,1] - T2*(1-probs.opt[,2])/probs.opt[,2]^2 - T3/probs.opt[,3] - T4/probs.opt[,4]))%*%X
-  X.G.1.5<-t(X*probs.opt[,2]*(T1/probs.opt[,1] + T2*(1-probs.opt[,2])/probs.opt[,2]^2 - T3/probs.opt[,3] + T4/probs.opt[,4]))%*%X
-  X.G.1.6<-t(X*probs.opt[,2]*(-T1/probs.opt[,1] - T2*(1-probs.opt[,2])/probs.opt[,2]^2 - T3/probs.opt[,3] + T4/probs.opt[,4]))%*%X
-  X.G.2.1<-t(X*probs.opt[,2]*probs.opt[,3])%*%X
-  X.G.2.2<-t(-X*probs.opt[,3]*(1-probs.opt[,3]))%*%X
-  X.G.2.3<-t(X*probs.opt[,3]*probs.opt[,4])%*%X
-  X.G.2.4<-t(X*probs.opt[,3]*(T1/probs.opt[,1] + T2/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]^2 - T4/probs.opt[,4]))%*%X
-  X.G.2.5<-t(X*probs.opt[,3]*(T1/probs.opt[,1] - T2/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]^2 + T4/probs.opt[,4]))%*%X
-  X.G.2.6<-t(X*probs.opt[,3]*(-T1/probs.opt[,1] + T2/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]^2 + T4/probs.opt[,4]))%*%X
-  X.G.3.1<-t(X*probs.opt[,2]*probs.opt[,4])%*%X
-  X.G.3.2<-t(X*probs.opt[,3]*probs.opt[,4])%*%X
-  X.G.3.3<-t(-X*probs.opt[,4]*(1-probs.opt[,4]))%*%X
-  X.G.3.4<-t(X*probs.opt[,4]*(T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] + T4*(1-probs.opt[,4])/probs.opt[,4]^2))%*%X
-  X.G.3.5<-t(X*probs.opt[,4]*(T1/probs.opt[,1] - T2/probs.opt[,2] - T3/probs.opt[,3] - T4*(1-probs.opt[,4])/probs.opt[,4]^2))%*%X
-  X.G.3.6<-t(X*probs.opt[,4]*(-T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] - T4*(1-probs.opt[,4])/probs.opt[,4]^2))%*%X
-  XW.1<-X*(T2-probs.opt[,2])
-  XW.2<-X*(T3-probs.opt[,3])
-  XW.3<-X*(T4-probs.opt[,4])
-  XW.4<-X*( T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] - T4/probs.opt[,4])
-  XW.5<-X*( T1/probs.opt[,1] - T2/probs.opt[,2] - T3/probs.opt[,3] + T4/probs.opt[,4])
-  XW.6<-X*(-T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] + T4/probs.opt[,4])
+  X.G.1.1<-t(-X*sample.weights*probs.opt[,2]*(1-probs.opt[,2]))%*%X
+  X.G.1.2<-t(X*sample.weights*probs.opt[,2]*probs.opt[,3])%*%X
+  X.G.1.3<-t(X*sample.weights*probs.opt[,2]*probs.opt[,4])%*%X
+  X.G.1.4<-t(X*sample.weights*probs.opt[,2]*(T1/probs.opt[,1] - T2*(1-probs.opt[,2])/probs.opt[,2]^2 - T3/probs.opt[,3] - T4/probs.opt[,4]))%*%X
+  X.G.1.5<-t(X*sample.weights*probs.opt[,2]*(T1/probs.opt[,1] + T2*(1-probs.opt[,2])/probs.opt[,2]^2 - T3/probs.opt[,3] + T4/probs.opt[,4]))%*%X
+  X.G.1.6<-t(X*sample.weights*probs.opt[,2]*(-T1/probs.opt[,1] - T2*(1-probs.opt[,2])/probs.opt[,2]^2 - T3/probs.opt[,3] + T4/probs.opt[,4]))%*%X
+  X.G.2.1<-t(X*sample.weights*probs.opt[,2]*probs.opt[,3])%*%X
+  X.G.2.2<-t(-X*sample.weights*probs.opt[,3]*(1-probs.opt[,3]))%*%X
+  X.G.2.3<-t(X*sample.weights*probs.opt[,3]*probs.opt[,4])%*%X
+  X.G.2.4<-t(X*sample.weights*probs.opt[,3]*(T1/probs.opt[,1] + T2/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]^2 - T4/probs.opt[,4]))%*%X
+  X.G.2.5<-t(X*sample.weights*probs.opt[,3]*(T1/probs.opt[,1] - T2/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]^2 + T4/probs.opt[,4]))%*%X
+  X.G.2.6<-t(X*sample.weights*probs.opt[,3]*(-T1/probs.opt[,1] + T2/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]^2 + T4/probs.opt[,4]))%*%X
+  X.G.3.1<-t(X*sample.weights*probs.opt[,2]*probs.opt[,4])%*%X
+  X.G.3.2<-t(X*sample.weights*probs.opt[,3]*probs.opt[,4])%*%X
+  X.G.3.3<-t(-X*sample.weights*probs.opt[,4]*(1-probs.opt[,4]))%*%X
+  X.G.3.4<-t(X*sample.weights*probs.opt[,4]*(T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] + T4*(1-probs.opt[,4])/probs.opt[,4]^2))%*%X
+  X.G.3.5<-t(X*sample.weights*probs.opt[,4]*(T1/probs.opt[,1] - T2/probs.opt[,2] - T3/probs.opt[,3] - T4*(1-probs.opt[,4])/probs.opt[,4]^2))%*%X
+  X.G.3.6<-t(X*sample.weights*probs.opt[,4]*(-T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] - T4*(1-probs.opt[,4])/probs.opt[,4]^2))%*%X
+  XW.1<-X*(T2-probs.opt[,2])*sample.weights^.5
+  XW.2<-X*(T3-probs.opt[,3])*sample.weights^.5
+  XW.3<-X*(T4-probs.opt[,4])*sample.weights^.5
+  XW.4<-X*( T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] - T4/probs.opt[,4])*sample.weights^.5
+  XW.5<-X*( T1/probs.opt[,1] - T2/probs.opt[,2] - T3/probs.opt[,3] + T4/probs.opt[,4])*sample.weights^.5
+  XW.6<-X*(-T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] + T4/probs.opt[,4])*sample.weights^.5
   G<-1/n*rbind(cbind(X.G.1.1,X.G.1.2,X.G.1.3,X.G.1.4,X.G.1.5,X.G.1.6),
                cbind(X.G.2.1,X.G.2.2,X.G.2.3,X.G.2.4,X.G.2.5,X.G.2.6),
                cbind(X.G.3.1,X.G.3.2,X.G.3.3,X.G.3.4,X.G.3.5,X.G.3.6))
@@ -699,7 +705,7 @@ CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, st
   
   class(beta.opt) <- "coef"
   
-  output<-list("coefficients"=beta.opt,"fitted.values"=probs.opt,"deviance"=deviance,"weights"=w.opt, 
+  output<-list("coefficients"=beta.opt,"fitted.values"=probs.opt,"deviance"=deviance,"weights"=sample.weights*w.opt, 
                "y"=treat,"x"=X,"converged"=opt1$conv,"J"=J.opt,"var"=vcov,   
                "mle.J"=ifelse(twostep, gmm.func(mcoef, invV = this.invV)$loss, gmm.loss(mcoef)))
   
